#include <system.h>
#include "interrupt.h"
unsigned char led[30];          // LED matrix, pwm values 0..15

unsigned char shadow = 0;       // ISR trisio shadow register
unsigned char colpos = 1;       // ISR gpio column ring counter bit
unsigned char ledndx;           // ISR led array index, 0..19
unsigned char dcy = 15;         // ISR duty cycle counter, 0..15
/********************************************************************/
/*  interrupt service routine                                       */
/********************************************************************/
void interrupt()
{ 
  unsigned char dc0;        // row0 (b0) duty cycle, 0..15
  unsigned char dc1;        // row1 (b1) duty cycle, 0..15
  unsigned char dc2;        // row2 (b2) duty cycle, 0..15
  unsigned char dc3;        // row3 (b4) duty cycle, 0..15
  unsigned char dc4;		// row4 (b5) duty cycle, 0..15
  asm
  {
    movf    _dcy,W          ; duty cycle counter, 0..15
    xorwf   _dc0,W          ; same as row0 LED duty cycle (b0)?
    btfsc   _status,Z       ; no, skip, else
    bcf     _shadow,0       ; clear row0 'on' bit in shadow reg'
    
    movf    _dcy,W          ;
    xorwf   _dc1,W          ; same as row1 LED duty cycle (b1)?
    btfsc   _status,Z       ;
    bcf     _shadow,1       ;
    
    movf    _dcy,W          ;
    xorwf   _dc2,W          ; same as row2 LED duty cycle (b2)?
    btfsc   _status,Z       ;
    bcf     _shadow,2       ;
    
    movf    _dcy,W          ;
    xorwf   _dc3,W          ; same as row3 LED duty cycle (b3)?
    btfsc   _status,Z       ;
    bcf     _shadow,3       ;
    
    movf	_dcy,W			;
    xorwf	_dc4,W			; same as row4 LED duty cycle (b4)?
    btfsc	_status,Z		;
	bcf		_shadow,4		;
    
    movf    _colpos,W       ;
    andwf   _shadow,W       ; is the float bit required?
    btfss   _status,Z       ; no, skip, else
    iorlw   0b00100000      ; set the 'float' bit
    iorwf   _shadow,W       ; pick up led bits
    iorwf   _colpos,W       ; pick up column bit
    xorlw   0b00111111      ; invert all
//  bsf     _status,RP0     ; compiler is doing this for me
    movwf   _trisb        ; update the column LEDs
//  bcf     _status,RP0     ; compiler is doing this for me
    incf    _dcy,F          ; increment duty cycle counter
    btfss   _dcy,4          ; all 16 pwm steps?  yes, skip, else
    goto    isrexit         ; exit isr
;
;  setup for next column
;
nextcol:
    clrf    _dcy            ; reset duty cycle counter
    bcf     _status,C       ;
    rlf     _colpos,F       ; shift column bit mask
    btfss   _colpos,6       ; all 5 columns scanned?
    goto    prepcol         ; no, branch, else
;
;  reset key variables if end of complete 5 column update
;
    movlw   0b00000001      ;
    movwf   _colpos         ; reset colpos bit
    movlw   _led            ;
    movwf   _ledndx         ; reset ledndx = &led[col 0 row 0]
;
;  setup shadow, GPIO output pattern, and load PWM counters
;
prepcol:
    movlw   0b00011111      ;
    movwf   _shadow         ; reset shadow (all output bits "on")
    movf    _colpos,W       ;
    movwf   _portb           ; portb = colpos, only 1 bit high
    movf    _ledndx,W       ;
    
    movwf   _fsr            ; fsr = &led[col,0]
    movf    _indf,W         ;
    movwf   _dc0            ; set row0 (b0) pwm, 0..15
    
    incf    _fsr,F          ; fsr = &led[col,1]
    movf    _indf,W         ;
    movwf   _dc1            ; set row1 (b1) pwm, 0..15
    
    incf    _fsr,F          ; fsr = &led[col,2]
    movf    _indf,W         ;
    movwf   _dc2            ; set row2 (b2) pwm, 0..15
    
    incf    _fsr,F          ; fsr = &led[col,3]
    movf    _indf,W         ;
    movwf   _dc3            ; set row3 (b3) pwm, 0..15
    
    incf	_fsr,F			;  fsr = &led[col,3]
    movf	_indf,W
    movwf	_dc4			;set row 5 (b4) pwm, 0..15
    
    incf    _fsr,W          ;
    movwf   _ledndx         ; save array index
isrexit:
	bcf		_intcon,TMR0IF		;clear timer flag
  }
}

void pwm_on(void)
{
	clear_bit(option_reg, T0CS);
	clear_bit(intcon,TMR0IF);
	set_bit(intcon, TMR0IE);
	set_bit(intcon, GIE);
	return;
}

void pwm_off(void)
{
	trisa = 0;
	porta = 0;
	trisb = 0;
	portb = 0;
	clear_bit(intcon, TMR0IE);
	clear_bit(intcon, TMR0IF);
	set_bit(option_reg, T0CS);

	return;
}
